/*
 * Grafo.java
 * Verso: 2004-08-26
 * Autores: Celmar Guimares da Silva e Petr Stukjunger
 */

package grafo;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.geom.Point2D;

/** Classe abstrata para representar grafos.
 */
public abstract class Grafo extends Observable
{
    
    /** Indica a rea externa ao retngulo que contm o grafo.
     */
    public final static int AREA_EXTERIOR = 0;
    
    /** Indica a rea interna ao retngulo em que o grafo est desenhado, exceto as
     * reas dos ns.
     */
    public final static int AREA_INTERIOR = 1;
    
    /** Indica a rea dos ns.
     */
    public final static int AREA_DE_NO    = 2;
    
    /** Cor padro de um n.
     */
    public static CorNo defaultCorNo = new CorNo();
    
    /** Cores de arestas.
     */
    public CorAresta corAresta = new CorAresta();
    
    /** Grupo padro (grupo a que todos os ns pertencem caso eles no tenham sido
     * colocados em nenhum grupo).
     */
    public static Grupo defaultGrupo = new Grupo("Default", defaultCorNo);
    
    /** Lista dos ns.
     */
    public java.util.List nos;
    
    /** Lista das arestas.
     */
    public java.util.Set arestas;
    
    /** Informa se grafo contm peso nos ns.
     */
    private boolean nosPossuemPeso = false;
    
    /** Informa se grafo contm peso nas arestas.
     */
    private boolean arestasPossuemPeso = false;
    
    /** Cor de fundo.
     */
    public Color corDeFundo;
    
    /** Lista dos grupos (tipos) de ns.
     */
    private java.util.List grupos;
    
    /** N que est sendo movido no momento.
     */
    private No noMovendo;
    
    /** Valor anterior de noMovendo.
     */
    private Point2D.Double posicaoAnteriorNoMovendo = new Point2D.Double(0,0);

    /** Indica a dimenso da tela com a qual se pode trabalhar. */
    private Dimension dimensaoDaTela;
    
    /** Cria uma nova instncia de Grafo.
     */
    public Grafo() {
        nos = new ArrayList();
        arestas = new HashSet();
        corDeFundo = new Color(0xe0e0e0); //Color.white;
        grupos = new ArrayList();
        noMovendo=null;
    }
 
    /** Faz com que o grafo leve em considerao os pesos associados aos ns.
     */    
    public void considerarPesosDosNos() {
        nosPossuemPeso = true;
    }
    
    /** Faz com que o grafo no leve em considerao os pesos associados aos ns.
     */    
    public void desconsiderarPesosDosNos() {
        nosPossuemPeso = false;
    }
    
    /** Faz com que o grafo leve em considerao os pesos associados s arestas.
     */    
    public void considerarPesosDasArestas() {
        arestasPossuemPeso = true;
    }
    
    /** Faz com que o grafo no leve em considerao os pesos associados s arestas.
     */    
    public void desconsiderarPesosDasArestas() {
        arestasPossuemPeso = false;
    }    
    
    /** Informa se o peso dos ns est sendo considerado.
     * @return True se o peso est sendo considerado, false caso contrrio.
     */    
    public boolean nosPossuemPeso() {
        return nosPossuemPeso;
    }
    
    /** Informa se o peso das arestas est sendo considerado.
     * @return True se o peso est sendo considerado, false caso contrrio.
     */    
    public boolean arestasPossuemPeso() {
        return arestasPossuemPeso;
    }
    /** Desenha o grafo.
     * @param g Onde o grafo ser desenhado.
     */
    public void desenhar(Graphics2D g) {
        Shape shape = g.getClip();
        
        // Desenhar arestas na ordem:
        // 1 - Arestas dos nos no selecionados. (Menos importantes)
        // 2 - Arestas dos ns selecionados. (Mais importantes)
        
        Iterator li = arestas.iterator();
        while (li.hasNext()) {
            Aresta aresta = (Aresta) li.next();
            if (!aresta.eAtingida() && !aresta.eEscondida()) {
                aresta.desenhar(this, g);
            }
        }
        
        li = arestas.iterator();
        while (li.hasNext()) {
            Aresta aresta = (Aresta) li.next();
            if (aresta.eAtingida() && !aresta.eEscondida()) {
                aresta.desenhar(this, g);
            }
        }
        
        // Desenhar ns na ordem:
        // 1 - Ns no selecionados (menos importantes)
        // 2 - Ns selecionados 
        // 3 - Ns marcados
        // 4 - N foco (mais importante)
        
        li = nos.listIterator();
        while (li.hasNext()) {
            No no = (No) li.next();
            if (!no.eAtingido() && !no.eMarcado()) {
                no.desenhar(this, g);
            }
        }
        
        li = nos.listIterator();
        while (li.hasNext()) {
            No no = (No) li.next();
            if (no.eAtingido() && !no.eMarcado()) {
                no.desenhar(this, g);
            }
        }
        
        li = nos.listIterator();
        while (li.hasNext()) {
            No no = (No) li.next();
            if (no.eMarcado()) {
                no.desenhar(this, g);
            }
        }
        
        if (noMovendo != null) {
            noMovendo.desenhar(this, g);
        }
        
        g.setClip(shape); // voltar forma antiga
        
    }    

    /** Indica se um ponto P informado est dentro de um n do grafo ou no
     * @param x Abscissa do ponto P
     * @param y Ordenada do ponto P
     * @return  Retorna AREA_DE_NO - se o ponto est sobre um n;
     *          AREA_INTERIOR - se est dentro do retngulo mas fora de um n;
     *          AREA_EXTERIOR - se est fora do retngulo.
     */    
    public int areaEm(int x, int y) {
        int resposta;
        if (dimensaoDaTela!=null) {
            if (x >= 0 && y>= 0 && x < dimensaoDaTela.width && y < dimensaoDaTela.height) {
                No no;
                resposta = AREA_INTERIOR;
                for (Iterator i = nos.iterator(); i.hasNext(); ) {
                    no = (No)i.next();
                    if (no.areaEm(x,y) == No.AREA_INTERIOR && !no.eEscondido()) {
                        resposta = AREA_DE_NO;
                    }
                }
                
            } else {
                resposta = AREA_EXTERIOR;
            }
        } else {
            resposta = AREA_EXTERIOR;
        }
        return resposta;
    }    

    /** Registra qual n est em movimento.
     * @param no N que est em movimento.
     */
    public void registrarNoMovendo(No no) {
        noMovendo = no;
        noMovendo.marcarComoMovendo();
        posicaoAnteriorNoMovendo.x = noMovendo.x;
        posicaoAnteriorNoMovendo.y = noMovendo.y;
    }
    
    /** Desfaz o registro de qual n est em movimento.
     */
    public void desfazerRegistrarNoMovendo() {
        if (noMovendo != null) {
            noMovendo.desmarcarComoMovendo();
            if (posicaoAnteriorNoMovendo.x != noMovendo.x || posicaoAnteriorNoMovendo.y != noMovendo.y) {
                registrarModificacaoDoGrafo();
            }
        }
        noMovendo = null;
    }
    
    /** Retorna qual o n que est em movimento.
     * @return Retorna o n que est em movimento, ou null se nenhum n estiver nessa situao.
     */
    public No retornarNoMovendo() {
        return noMovendo;
    }
    
     /** Mover n que est em movimento para a posio nova (x,y).
     * @param x Abscissa da nova posio.
     * @param y Ordenada da nova posio.
     */
    public void moverNoMovendo(int x, int y) {
        // ajusta para que o n no possa ser movido para fora do retngulo do
        // grafo
        int nx = x, ny = y; // posio do mouse
        if (x < 0) {
            nx = 0;
        }
        if (x > dimensaoDaTela.width) {
            nx = (int) dimensaoDaTela.width;
        }
        if (y < 0) {
            ny = 0;
        }
        if (y > dimensaoDaTela.height) {
            ny = (int) dimensaoDaTela.height;
        }
        // aqui nx e ny j tem a posio do mouse, ajustada para estar dentro
        // do retngulo
        
        // ajustando posio do noMovendo
        noMovendo.x = nx;
        noMovendo.y = ny;
    }

    /** Verifica qual o n sob um determinado ponto (x,y).
     * @param x Abscissa do ponto.
     * @param y Ordenada do ponto.
     * @return  Retorna o n, ou null se o n no existir.
     */
    public No noEm(int x, int y) {
        /* Confere n a n, na sequncia inversa da que as arestas
        desenhadas, se o n contm o ponto x,y .*/
        //nos.toArray();
        
        No no;
        No resposta = null;
        
        // primeiro, os ns atingidos
        for (int i = nos.size()-1; i>=0 && resposta==null; i--) {
            no = (No) nos.get(i);
            if (no.eAtingido() && !no.eEscondido()) {
                if (no.areaEm(x,y) == No.AREA_INTERIOR) {
                    resposta = no;
                }
            }
        }
        
        if (resposta==null) {
            // depois, os outros.
            for (int i = nos.size()-1; i>=0 && resposta==null; i--) {
                no = (No) nos.get(i);
                if (!no.eAtingido() && !no.eEscondido()) {
                    if (no.areaEm(x,y) == No.AREA_INTERIOR) {
                        resposta = no;
                    }
                }
            }
        }
        return resposta;
    }
    
    
    /** Adiciona ao grafo um n especificado.
     * @param no N a ser adicionado.
     */
    public void adicionarNo(No no) {
        adicionarNo(no, true);
    }
        
    /** Adiciona ao grafo um n especificado.
     * @param no N a ser adicionado.
     * @param registrarModificacao Informa se a ao de adicionar o n deve ser 
     * considerada como modificao.
     */    
    synchronized public void adicionarNo(No no, boolean registrarModificacao) {        
        no.desmarcar();
        nos.add(no);
        if (registrarModificacao) {
            registrarModificacaoDoGrafo();
        }
    }

    /** Adiciona ao grafo um conjunto de ns especificados. 
     * @param colecaoDeNos Conjunto de ns a serem adicionados.
     */    
    public void adicionarNos(Collection colecaoDeNos) {
        adicionarNos(colecaoDeNos, true);
    }
    
    /** Adiciona ao grafo um conjunto de ns especificados. 
     * @param colecaoDeNos Conjunto de ns a serem adicionados.
     * @param registrarModificacao Informa se a ao de adicionar esses ns deve 
     * ser considerada como modificao.
     */    
    synchronized public void adicionarNos(Collection colecaoDeNos, boolean registrarModificacao) {
        Iterator i = colecaoDeNos.iterator();
        No n;
        while (i.hasNext()) {
            n = (No)i.next();
            adicionarNo(n, false);
        }
        if (registrarModificacao) {
            registrarModificacaoDoGrafo();
        }
    }
    
    /** Adiciona ao grafo uma aresta sem peso e sem informao.
     * @param no1 Primeiro n da aresta a ser adicionada.
     * @param no2 Segundo n da aresta a ser adicionada.
     */
    synchronized public void adicionarAresta(No no1, No no2) {
        adicionarAresta(no1, no2, 0, null);
    }
    
    /** Adiciona ao grafo uma aresta sem informao.
     * @param no1 Primeiro n da aresta a ser adicionada.
     * @param no2 Segundo n da aresta a ser adicionada.
     * @param peso Peso da aresta a ser adicionada.
     */
    synchronized public void adicionarAresta(No no1, No no2, int peso) {
        adicionarAresta(no1, no2, peso, null);
    }
    
    /** Adiciona ao grafo uma aresta sem peso.
     * @param no1 Primeiro n da aresta a ser adicionada.
     * @param no2 Segundo n da aresta a ser adicionada.
     * @param info Informao sobre a aresta a ser adicionada.
     */
    synchronized public void adicionarAresta(No no1, No no2, Info info) {
        adicionarAresta(no1, no2, 0, info);
    }
    
    /** Adiciona ao grafo uma aresta com peso e informao.
     * @param no1 Primeiro n da aresta a ser adicionada.
     * @param no2 Segundo n da aresta a ser adicionada.
     * @param peso Peso da aresta a ser adicionada.
     * @param info Informao sobre a aresta a ser adicionada.
     */
    public void adicionarAresta(No no1, No no2, int peso, Info info) {
        adicionarAresta(no1,no2,peso,info,true);
    }
    
    /** Adiciona ao grafo uma aresta com peso e informao.
     * @param no1 Primeiro n da aresta a ser adicionada.
     * @param no2 Segundo n da aresta a ser adicionada.
     * @param peso Peso da aresta a ser adicionada.
     * @param info Informao sobre a aresta a ser adicionada.
     * @param registrarModificacao Informa se a ao de adicionar a aresta deve 
     * ser considerada como modificao do grafo ou no.
     */
    public abstract void adicionarAresta(No no1, No no2, int peso, Info info, boolean registrarModificacao);

    /** Adiciona ao grafo uma aresta informada.
     *  Observaes
     *  - No verifica se essa aresta j existe.
     *  - Caso um n na extremidade da aresta no exista na lista de ns,
     *  insere-o nessa lista.
     * @param aresta Aresta a ser adicionada ao grafo.
     */
    public void adicionarAresta(Aresta aresta) {
        adicionarAresta(aresta, true);
    }
    
    /** Adiciona ao grafo uma aresta informada.
     *  Observaes
     *  - No verifica se essa aresta j existe.
     *  - Caso um n na extremidade da aresta no exista na lista de ns,
     *  insere-o nessa lista.
     * @param aresta Aresta a ser adicionada ao grafo.
     * @param registrarModificacao Informa se a ao de adicionar a aresta deve ser considerada como modificao
     * do grafo ou no.
     */
    synchronized public void adicionarAresta(Aresta aresta, boolean registrarModificacao) {
        if (aresta!=null) {
            aresta.no1.arestas.add(aresta);
            if (aresta.no1 != aresta.no2) {// evita adicionar uma aresta duas vezes ao mesmo n.
                aresta.no2.arestas.add(aresta);
            }
            
            // Adiciona n se ele ainda no est na lista de ns do grafo.
            if (!conterNo(aresta.no1)) {
                adicionarNo(aresta.no1);
            }
            if (!conterNo(aresta.no2)) {
                adicionarNo(aresta.no2);
            }
            if (aresta.no1.eEscondido() || aresta.no2.eEscondido()) {
                aresta.esconder();
            } 
            if (aresta.no1.eMarcado() || aresta.no2.eMarcado()) {
                aresta.atingir();
            }
            
            arestas.add(aresta);
            if (aresta.peso!=0 && !arestasPossuemPeso) {
                considerarPesosDasArestas();
            }
            if (registrarModificacao) {
                registrarModificacaoDoGrafo();
            }
        }

    }

    /** Adiciona ao grafo um conjunto de arestas informadas.
     * @param colecaoDeArestas Arestas a serem adicionadas ao grafo.
     */
    public void adicionarArestas(Collection colecaoDeArestas) {
        adicionarArestas(colecaoDeArestas, true);
    }
    
    /** Adiciona ao grafo um conjunto de arestas informadas.
     * @param colecaoDeArestas Arestas a serem adicionadas ao grafo.
     * @param registrarModificacao Informa se a ao de adicionar essas arestas
     * deve ser considerada como modificao do grafo ou no.
     */
    synchronized public void adicionarArestas(Collection colecaoDeArestas, boolean registrarModificacao) {
        Iterator i = colecaoDeArestas.iterator();
        Aresta a;
        while (i.hasNext()) {
            a = (Aresta)i.next();
            adicionarAresta(a, false);
        }
        if (registrarModificacao) {
            registrarModificacaoDoGrafo();
        }
    }

    /** Remove do grafo um n especificado e as arestas conectadas a ele.
     * @param no N a ser removido.
     */
    public void removerNo(No no) {
        removerNo(no, true);
    }
        
    /** Remove do grafo um n especificado e as arestas conectadas a ele.
     * @param no N a ser removido.
     * @param registrarModificacao Informa se a ao de remover esse n
     * deve ser considerada como modificao do grafo ou no.
     */
    synchronized public void removerNo(No no, boolean registrarModificacao) {
        // remover n
        no.desmarcar();
        if (nos.remove(no)) {
            // remover arestas conectadas ao n
            Iterator li = arestas.iterator();
            while (li.hasNext()) {
                Aresta aresta = (Aresta) li.next();
                if (aresta.no1 == no || aresta.no2 == no) {
                    // remover aresta da lista nos ns
                    
                    aresta.no1.arestas.remove(aresta);
                    if (aresta.no1 != aresta.no2) {
                        aresta.no2.arestas.remove(aresta);
                    }
                    // remover aresta da lista em grafo
                    li.remove(); // nao use arestas.remove!!! <- problemas com sincronizao (Petr)
                    
                }
            }
            if (registrarModificacao) {
                registrarModificacaoDoGrafo();
            }
        }
    }

    /**
     * Remove do grafo um conjunto de ns especificados e as arestas conectadas
     * a eles.
     * @param colecaoDeNos Ns a serem removidos.
     */
    public void removerNos(Collection colecaoDeNos) {
        removerNos(colecaoDeNos, true);
    }
    
    /** Remove do grafo um conjunto de ns especificados e as arestas conectadas
     * a eles.
     * @param colecaoDeNos Ns a serem removidos.
     * @param registrarModificacao Informa se a ao de remover esses ns
     * deve ser considerada como modificao do grafo ou no.
     */
    synchronized public void removerNos(Collection colecaoDeNos, boolean registrarModificacao) {
        Iterator i = colecaoDeNos.iterator();
        No n;
        while (i.hasNext()) {
            n = (No)i.next();
            removerNo(n, false);
        }
        if (registrarModificacao) {
            registrarModificacaoDoGrafo();
        }
    }


    /** Remove do grafo uma aresta especificada.
     * @param aresta Aresta a ser removida.
     */
    public void removerAresta(Aresta aresta) {
        removerAresta(aresta, true);
    }
    
    /** Remove do grafo uma aresta especificada.
     * @param aresta Aresta a ser removida.
     * @param registrarModificacao Informa se a ao de remover essa aresta
     * deve ser considerada como modificao do grafo ou no.
     */
    synchronized public void removerAresta(Aresta aresta, boolean registrarModificacao) {
        if (arestas.remove(aresta)) {
            aresta.desfazerAtingir();

            aresta.esconder();            
            // remover aresta da lista de arestas de cada n
            aresta.no1.arestas.remove(aresta);
            if (aresta.no1 != aresta.no2) {
                aresta.no2.arestas.remove(aresta);
            }
            if (registrarModificacao) {
                registrarModificacaoDoGrafo();
            }
        } else {
            System.out.println("ERRO: Aresta no encontrada. Impossivel remover aresta.");
            // excecao...
        }
    }

    /** Remove do grafo um conjunto de arestas especificadas.
     * @param colecaoDeArestas Arestas a serem removidas.
     */
    public void removerArestas(Collection colecaoDeArestas) {
        removerArestas(colecaoDeArestas, true);
    }

    /** Remove do grafo um conjunto de arestas especificadas.
     * @param colecaoDeArestas Arestas a serem removidas.
     * @param registrarModificacao Informa se a ao de remover essas arestas
     * deve ser considerada como modificao do grafo ou no.
     */
    synchronized public void removerArestas(Collection colecaoDeArestas, boolean registrarModificacao) {
        Iterator i = colecaoDeArestas.iterator();
        Aresta a;
        while (i.hasNext()) {
            a = (Aresta)i.next();
            removerAresta(a, false);
        }
        if (registrarModificacao) {
            registrarModificacaoDoGrafo();
        }
    }

    /** Informa se um n determinado pertence ao grafo.
     * @param no N a ser procurado.
     * @return True se o n existe na lista de ns do grafo, false caso
     * contrrio.
     */
    public boolean conterNo(No no) {
        return nos.indexOf(no) != -1 ;
    }
    
    /** Informa se a aresta pertence ao grafo.
     * @param aresta Aresta a ser procurada.
     * @return True se a aresta existe no grafo, false caso contrrio.
     */
    public boolean conterAresta(Aresta aresta) {
        return arestas.contains(aresta);
    }
    
    /** Retorna um n do grafo que contenha uma determinada informao.
     * @param info Informao a ser procurada.
     * @return Retorna o primeiro n que for encontrado contendo essa
     * informao, ou null caso no exista tal n.
     */
    public No buscarNo(Info info) {
        ListIterator li = nos.listIterator();
        No resposta = null;
        No no;
        while (li.hasNext() && resposta == null) {
            no = (No) li.next();
            if (no.info != null && no.info.equals(info)) {
                resposta = no;
            }
        }
        return null;
    }
    
    /** Retorna uma aresta do grafo que contenha uma determinada informao.
     * @param info Informao a ser procurada.
     * @return Retorna a primeira aresta que for encontrada contendo essa
     * informao, ou null caso no exista tal aresta.
     */
    public Aresta buscarAresta(Info info) {
        Iterator li = arestas.iterator();
        Aresta resposta = null;
        Aresta aresta;
        while (li.hasNext() && resposta==null) {
            aresta = (Aresta)li.next();
            if (aresta.info != null && aresta.info.equals(info)) {
                resposta=aresta;
            }
        }
        return resposta;
    }
    
    /** Retorna uma aresta do grafo que ligue os ns no1 e no2.
     * @param no1 N da aresta.
     * @param no2 N da aresta.
     * @return Retorna a primeira aresta que for encontrada ligando esses ns,
     * ou null caso no exista tal aresta.
     */
    public Aresta buscarAresta(No no1, No no2) {
        Aresta resposta = null;
        if (no1 != null && no2 != null) {
            Iterator li = no1.arestas.iterator();
            while (li.hasNext() && resposta == null) {
                Aresta aresta = (Aresta) li.next();
                if ( (aresta.no1 == no1 && aresta.no2 == no2) ||
                (aresta.no1 == no2 && aresta.no2 == no1) ) {
                    resposta = aresta;
                }
            }
        }
        return resposta;
        // Obs: Esse algoritmo pode ser transformado em O(1). Hoje  O(n).
        // Para ser transformado em O(1),  preciso que this.arestas seja um 
        // hashMap indexado segundo um par de ns (os ns que compem cada 
        // aresta).
    }
    
    
    /** Adiciona um grupo (categoria/tipo) de ns ao grafo. No adiciona nenhum
     * n.
     * @param grupo Grupo a ser adicionado.
     */
    public void adicionarGrupo(Grupo grupo) {
        grupos.add(grupo);
    }
    
    /** Remove um grupo (categoria/tipo) de ns ao Grafo. No remove nenhum n.
     * @param grupo Grupo a ser removido.
     */
    public void removerGrupo(Grupo grupo) {
        grupos.remove(grupos.indexOf(grupo));
    }
    
    /** Retorna um ListIterator com os grupos existentes no grafo.
     * @return Lista de grupos do grafo.
     */
    public Iterator retornarGrupos() {
        return grupos.iterator();
    }

    /** Retorna a quantidade de grupos existentes no grafo.
     * @return Quantidade de grupos.
     */
    public int retornarQuantidadeDeGrupos() {
        return grupos.size();
    }
    
    /** Procura um grupo com o nome informado.
     * @param nome Nome do grupo a ser procurado.
     * @return Retorna o grupo, se exisitr, ou null, caso contrrio.
     */    
    public Grupo retornarGrupo(String nome) {
        ListIterator li = grupos.listIterator();
        boolean achei = false;
        Grupo gr = null;
        while (li.hasNext() && !achei) {
            gr = (Grupo)li.next();
            achei = (gr.nome.equals(nome));
        }
        if (!achei) {
            gr = null;
        }
        return gr;
    }

    /** Retorna a quantidade de arestas existentes no grafo.
     * @return Quantidade de grupos.
     */
    public int retornarQuantidadeDeArestas() {
        return arestas.size();
    }

    /** Retorna a quantidade de ns existentes no grafo.
     * @return Quantidade de grupos.
     */
    public int retornarQuantidadeDeNos() {
        return nos.size();
    }
    
    /** Retorna um ListIterator com os ns existentes no grafo.
     * @return Lista de grupos do grafo.
     */
    public Iterator retornarNos() {
        return nos.iterator();
    }
    
    /** Esconde um n do grafo.
     * @param no N a ser escondido.
     */
    public void esconderNo(No no) {
        esconderNo(no, true);
    }
        
    /** Esconde um n do grafo.
     * @param no N a ser escondido.
     * @param registrarModificacao Informa se a ao de esconder o n deve ser
     * considerada como modificao do grafo ou no.
     */
    public void esconderNo(No no, boolean registrarModificacao) {
        if (no.eMovendo()) {
            return; //  muito dificil para esconder n que est em movimento
        }
        // remover n de lista em grafo
        int indexNo = nos.indexOf(no);
        if (indexNo!=-1 && !no.eEscondido()) { // se n pertence  lista de ns, ento posso escond-lo.
            if (no.eMarcado()) {
                no.desmarcar();
            }
            if (no.eEscolhido()) {
                no.desfazerEscolher();
            }
            no.esconder();
        }
        if (registrarModificacao) {
            registrarModificacaoDoGrafo();
        }
    }
    
    /** Esconde uma lista de ns e suas respectivas arestas.
     * @param colecaoDeNos Conjunto de ns a ser mostrado.
     */
    public void esconderNos(Collection colecaoDeNos) {
        esconderNos(colecaoDeNos, true);
    }
    
    /** Esconde uma lista de ns e suas respectivas arestas.
     * @param colecaoDeNos Conjunto de ns a ser mostrado.
     * @param registrarModificacao Informa se a ao de esconder esses ns
     * deve ser considerada como modificao do grafo ou no.
     */
    public void esconderNos(Collection colecaoDeNos, boolean registrarModificacao) {
        Iterator i = colecaoDeNos.iterator();
        No n;
        while (i.hasNext()) {
            n = (No)i.next();
            esconderNo(n, false);
        }
        if (registrarModificacao) {
            registrarModificacaoDoGrafo();
        }
    }
    
    /** Torna visvel um n que havia sido escondido.
     * @param no N a ser mostrado.
     */
    public void mostrarNo(No no) {
        mostrarNo(no, true);
    }
    
    /** Torna visvel um n que havia sido escondido.
     * @param no N a ser mostrado.
     * @param registrarModificacao Informa se a ao de esconder o n deve ser
     * considerada como modificao do grafo ou no.
     */
    public void mostrarNo(No no, boolean registrarModificacao) {
        no.mostrar();
        if (registrarModificacao) {
            registrarModificacaoDoGrafo();
        }
    }

    /** Torna visvel uma lista de ns e suas respectivas arestas.
     * @param colecaoDeNos Conjunto de ns a ser mostrado.
     */
    public void mostrarNos(Collection colecaoDeNos) {
        mostrarNos(colecaoDeNos,true);
    }
    
    /** Torna visvel uma lista de ns e suas respectivas arestas.
     * @param colecaoDeNos Conjunto de ns a ser mostrado.
     * @param registrarModificacao Informa se a ao de mostrar esses ns
     * deve ser considerada como modificao do grafo ou no.
     */
    public void mostrarNos(Collection colecaoDeNos, boolean registrarModificacao) {
        Iterator i = colecaoDeNos.iterator();
        No n;
        while (i.hasNext()) {
            n = (No)i.next();
            mostrarNo(n, false);
        }
        if (registrarModificacao) {
            registrarModificacaoDoGrafo();
        }
    }
    
    /** Mostra todos os ns que estavam escondidos.
     */
    public void mostrarTodosOsNos() {
        mostrarTodosOsNos(true);
    }
        
    /** Mostra todos os ns que estavam escondidos.
     * @param registrarModificacao Informa se a ao de mostrar esses ns
     * deve ser considerada como modificao do grafo ou no.
     */
    public void mostrarTodosOsNos(boolean registrarModificacao) {
        Object[] a = nos.toArray();
        for(int i = 0; i < a.length; i++) {
            if (((No)a[i]).eEscondido()) {
                mostrarNo((No)a[i], false);
            }
        }
        if (registrarModificacao) {
            registrarModificacaoDoGrafo();        
        }
    }
    
    /** Retorna uma lista de todos os ns escondidos.
     * @return Lista contendo ns escondidos.
     */
    public Collection retornarEscondidos() {
        Collection c = new ArrayList();
        Iterator i = nos.iterator();
        while(i.hasNext()) {
            No no = (No)i.next();
            if (no.eEscondido()) {
                c.add(no);
            }
        }
        return c;
    }
    
    /** Retorna uma lista de todos os ns visveis marcados (escolhidos).
     * @return Lista contendo ns marcados.
     */
    public Collection retornarEscolhidos() {
        Collection c = new ArrayList();
        Iterator i = nos.iterator();
        while(i.hasNext()) {
            No no = (No)i.next();
            if (no.eMarcado() && !no.eEscondido()) {
                c.add(no);
            }
        }
        return c;
    }
    
    /** Retorna uma lista de todos os ns visveis no marcados (no escolhidos).
     * @return Lista de ns no marcados.
     */
    public Collection retornarNaoEscolhidos() {
        Collection c = new ArrayList();
        Iterator i = nos.iterator();
        while(i.hasNext()) {
            No no = (No)i.next();
            if (!no.eMarcado() && !no.eEscondido()) {
                c.add(no);
            }
        }
        return c;
    }
    
    /** Algoritmo quadrtico ( O(n^2), n = nmero de ns ) para descobrir o conjunto de
     * arestas que passam por um ponto M=(xm,ym) no grafo.
     * @param xm Abscissa de M.
     * @param ym Ordenada de M.
     * @return Retorna o conjunto de arestas do grafo que passam pelo ponto M.
     */
    private Collection retornarConjuntoArestasEmQuadratico(int xm, int ym) {
        Collection conjuntoArestas = new HashSet();
        Aresta a;
        No no1, no2;
        Iterator i = arestas.iterator();
        while (i.hasNext()) {
            a = (Aresta)i.next();
            if (!a.eEscondida()) {
                no1 = (No)(a.no1);
                no2 = (No)(a.no2);
                if (a.eAtingida() && pontoEstaProximoDaAresta(xm,ym,no1.x, no1.y, no2.x, no2.y)) {
                    conjuntoArestas.add(a);
                }
            }
        }
        return (conjuntoArestas);
    }

    /** Algoritmo quadratico (O(n^2), n = nmero de ns) para descobrir o conjunto de
     * auto-arestas (arestas cujas extremidades incidem sobre o mesmo n) que passam
     * pelo ponto M=(xm,ym).
     * @param xm Abscissa de M.
     * @param ym Ordenada de M.
     * @return Retorna o conjunto de auto-arestas do grafo que passam pelo ponto M.
     */
    public Collection retornarConjuntoAutoArestasEm(int xm, int ym) {
        HashSet conjuntoArestas = new HashSet();
        Aresta aresta;
        double rm;
        final int delta = 3;
        Iterator it = arestas.iterator();
        while (it.hasNext()) {
            aresta = (Aresta)it.next();
            if (aresta.no1.equals(aresta.no2)) {
                Point centroAutoAresta = aresta.retornarCentroAutoAresta(aresta.no1, this);
                rm = Point.distance(centroAutoAresta.x,centroAutoAresta.y,xm,ym);
                if (aresta.raioArestaCircular-delta <= rm && rm <=aresta.raioArestaCircular+delta) {
                    if (!aresta.eEscondida() && aresta.eAtingida()) {
                        conjuntoArestas.add(aresta);
                    }
                }
            }
        }
        return (conjuntoArestas);
    }
    
    /** Verifica se o ponto M = (xm,ym) est prximo ao segmento de reta
     * AB = ((xa,ya),(xb,yb)).
     * @param xm Abscissa do ponto M.
     * @param ym Ordenada do ponto M.
     * @param xa Abscissa do ponto A.
     * @param ya Ordenada do ponto A.
     * @param xb Abscissa do ponto B.
     * @param yb Ordenada do ponto B.
     * @return True se o ponto est prximo do segmento de reta AB, false caso contrrio.
     */
    public boolean pontoEstaProximoDaAresta(int xm, int ym, double xa, double ya, double xb, double yb) {
        double dab, dam, dmb, dap, dbp, thetaA, thetaM, d;
        double dmax=5;
        boolean resposta = false;
        dab = Math.sqrt(Math.pow(xa-xb,2) + Math.pow(ya-yb,2));
        dam = Math.sqrt(Math.pow(xa-xm,2) + Math.pow(ya-ym,2));
        dmb = Math.sqrt(Math.pow(xm-xb,2) + Math.pow(ym-yb,2));
        thetaA = Math.toDegrees(Math.asin((ya-yb)/dab));
        if (xa>xb) { thetaA = 180 - thetaA; }
        thetaA = Util.normalizarAngulo(thetaA);
        thetaM = Math.toDegrees(Math.asin((ym-yb)/dmb));
        if (xm>xb) { thetaM = 180 - thetaM; }
        thetaM = Util.normalizarAngulo(thetaM);
        d = dmb * Math.sin(Math.toRadians(thetaA-thetaM));
        if (Math.abs(d) < dmax) {
            // M est prximo da aresta a. Mas P pertence a ela?
            dap = Math.sqrt(Math.pow(dam,2) - Math.pow(d,2));
            dbp = Math.sqrt(Math.pow(dmb,2) - Math.pow(d,2));
            resposta = (Math.abs(dap+dbp-dab)<1);
        }
        return resposta;
    }
    
    /** Limpa o grafo, removendo todos os seus ns, arestas e grupos.
     */
    synchronized public void limparGrafo() {
        nos.clear();
        arestas.clear();
        grupos.clear();
        registrarModificacaoDoGrafo();
    }

    /** Remove todas as arestas do grafo.
     */
    public void removerTodasAsArestas() {    
        removerTodasAsArestas(true);
    }
    
    /** Remove todas as arestas do grafo.
     * @param registrarModificacao Informa se a ao de remover essas arestas
     * deve ser considerada como modificao do grafo ou no.
     */
    synchronized public void removerTodasAsArestas(boolean registrarModificacao) {
        Iterator li = arestas.iterator();
        while (li.hasNext()) {
            ((Aresta)li.next()).esconder();
        }
        arestas.clear();
        li = nos.iterator();
        No n;
        while (li.hasNext()) {
            n = (No)li.next();
            n.arestas.clear();
            n.anularAtingir();
        }
        if (registrarModificacao) {
            registrarModificacaoDoGrafo();
        }
    }    
    
    /** Retorna o conjunto de arestas em um ponto M = (xm,ym)
     * @param xm Abscissa do ponto M.
     * @param ym Ordenada do ponto M.
     * @return Conjunto de arestas que passam pelo ponto M.
     */
    public Collection retornarConjuntoArestasEm(int xm, int ym) {
        Collection resposta;
        resposta = retornarConjuntoArestasEmQuadratico(xm, ym);
        resposta.addAll(retornarConjuntoAutoArestasEm(xm,ym));
        return(resposta);
    }
 
    /** Registra a ocorrncia de uma modificao no grafo, notificando seus
     * observadores (Observers).
     */
    public void registrarModificacaoDoGrafo() {
        setChanged();
        notifyObservers();
    }
    
    /** Atualiza o indicador da dimenso da tela.
     * @param d Dimenso atual da tela com a qual se pode trabalhar para desenhar o grafo.
     */
    public void ajustarDimensaoDaTela(Dimension d) {
        //DebugMessenger.showMessage("Nova dimensao = "+d);
        dimensaoDaTela=d;
        setChanged();
        notifyObservers();
    }

    /** Retorna a dimenso atualmente considerada pelo grafo como limite de 
     * desenho. 
     * @return Dimenso considerada.
     */
    public Dimension retornarDimensaoDaTela() {
        //DebugMessenger.showMessage("Usando dimensao atual como = "+dimensaoDaTela);
        return dimensaoDaTela;
    }

    /** Calcula o peso mximo do conjunto atual de arestas.
     * @return Retorna o peso mximo do conjunto atual de arestas; 0 se o 
     * conjunto atual for vazio.
     */
    public int retornarPesoMaximoArestas() {
        int max = 0;
        Aresta a;
        boolean primeiro = true;
        for (Iterator it = arestas.iterator(); it.hasNext(); ) {
            a = (Aresta)it.next();
            if (!a.eEscondida()) {
                if (primeiro) {
                    max = a.peso;
                    primeiro = false;
                } else if (max < a.peso) {
                    max = a.peso;
                }
            }
        }
        return max;
    }
    
    /** Calcula o peso mnimo do conjunto atual de arestas.
     * @return Retorna o peso mximo do conjunto atual de arestas; 0 se o 
     * conjunto atual for vazio.
     */
    public int retornarPesoMinimoArestas() {
        int min=0;
        Aresta a;
        boolean primeiro = true;
        for (Iterator it = arestas.iterator(); it.hasNext(); ) {
            a = (Aresta)it.next();
            if (!a.eEscondida()) {
                if (primeiro) {
                    min = a.peso;
                    primeiro = false;
                } else if (min > a.peso) {
                    min = a.peso;
                }
            }
        }
        return min;
    }

//    /** Mtodo para depurao. Mostra na sada padro uma lista com todos os ns do
//     * ArrayList nos.
//     */
//    private void mostrarListaNos() {
//        Iterator li = nos.iterator();
//        while (li.hasNext()) {
//            No n = (No)li.next();
//            System.out.print(n.nome + ",");
//        }
//        System.out.println();
//    }    
    
}