/*
 * GrafoPanel.java
 * Verso: 2004-07-04
 * Autores: Celmar Guimares da Silva e Petr Stukjunger
 */

package grafo;
import javax.swing.*;
import java.io.*;
import java.awt.event.*;
import java.awt.Dimension;
import java.util.List;

/** Cria um panel em que o grafo e seus respectivos controles so desenhados. */
public abstract class GrafoPanel extends JPanel
implements ComponentListener 
{
    
    /** Grafo a ser desenhado. */    
    private Grafo grafo;
    
    /** Display em que o grafo  mostrado. */    
    private GrafoDisplay display;
    
    /** Textos a serem utilizados pelo panel e pelo seu display. */
    public String[] textos;
    
    /** Cria uma nova instncia GrafoPanel.
     * @param grafo Grafo a ser mostrado pelo objeto.
     */
    public GrafoPanel(Grafo grafo) {
        definirTextosDefault();
        
        //usarGrafo(grafo);
        this.grafo = grafo;
        
        // definindo a interface
        display = criarDisplay(grafo);
        if (display!=null) {
            
            display.setAlignmentX(0.0F);
            display.setAlignmentY(0.0F);
            inserirLayoutEControles(display, grafo);
            
        } else {
            System.out.println("Erro. Display no pde ser inicializado.");
            System.out.println("Error. Display could not be initialized.");            
        }
        addComponentListener(this);

    }

    /**
     * Cria e insere layout e controles (se existirem) no GrafoPanel.
     * @param display Display em que o grafo ser mostrado.
     * @param grafo Grafo a ser mostrado.
     */    
    public abstract void inserirLayoutEControles(GrafoDisplay display, Grafo grafo);
    
    /**
     * Cria um display para mostrar o grafo informado.
     * @param grafo Grafo a ser mostrado.
     * @return Display, cujo tipo varia de acordo com o tipo do grafo informado.
     */    
    public abstract GrafoDisplay criarDisplay(Grafo grafo);
    
    /** Modifica textos utilizados em menus e dilogos. til para definir um novo
     * conjunto de textos, como seria necessrio no caso de textos em outro idioma.
     * @param textosNovos Textos que iro substituir os 
     * textos atuais, tanto no panel quanto em seu display.
     */
    public void modificarTextos(List textosNovos) {
        // O GrafoPanel em si no utiliza nenhum texto.
        // Logo, repassa para o display todos os textos recebidos.
        if (textosNovos!=null) {
            if (textosNovos.size() >= textos.length) {
                List textosNovosDoPanel = textosNovos.subList(0, textos.length);
                textos = (String[])textosNovosDoPanel.toArray(textos);
                if (display!=null) {
                    List textosNovosDoDisplay = textosNovos.subList(textos.length, textosNovos.size());
                    display.modificarTextos(textosNovosDoDisplay);
                } else {
                    System.out.println("Erro. No foi possvel modificar os textos do display, porque o display do grafo ainda no estava pronto.");
                    // System.out.println("Error. The display texts could not be modified because the graph display was not ready.");
                }
            } else {
                System.out.println("Erro. No foi possvel modificar os textos do panel, porque a lista contendo os novos textos  de tamanho diferente da lista original de textos.");
            }
        } else {
            System.out.println("No foi possvel modificar os textos do panel, porque a lista contendo os novos textos  nula.");
        }
        
    }

    /** Modifica o nome que descreve o peso dos ns. Inicialmente esse nome  "Peso",
     * mas pode haver interesse em modific-lo para especificar que esse peso se refere
     * a uma determinada medida (por exemplo, nmero de mensagens recebidas).
     * Esse nome  independente do nome do peso das arestas.
     * @param nome Novo nome a ser utilizado.
     */        
    public void renomearPesoDosNosPara(String nome) {
        display.renomearPesoDosNosPara(nome);
    }
    
    /** Modifica o nome que descreve o peso das arestas. Inicialmente esse nome  "Peso",
     * mas pode haver interesse em modific-lo para especificar que esse peso se refere
     * a uma determinada medida (por exemplo, nmero de mensagens representadas pela
     * aresta).
     * Esse nome  independente do nome do peso dos ns.
     * @param nome Novo nome a ser utilizado.
     */    
    public void renomearPesoDasArestasPara(String nome) {
        display.renomearPesoDasArestasPara(nome);
    }
    
    
    /** Salva um arquivo contendo a imagem do display mostrando o grafo.
     * @param nomeArquivo Nome do arquivo
     * @param tipoArquivo Tipo do arquivo (ao menos jpeg e png so suportados).
     * @return True se a operao foi bem sucedida, False caso contrrio.
     */    
    public boolean salvarDisplay(String nomeArquivo, String tipoArquivo) {
        boolean resposta = false;
        if (display!=null) {
            File arquivo = new File(nomeArquivo);
            resposta = display.salvarImagem(arquivo, tipoArquivo);
        } 
        return resposta;
    }

    /**
     * Informa se j foi definido algum display para este GrafoPanel.
     * @return True se um display foi definido, False caso contrrio.
     */    
    public boolean displayEstaDefinido() {
        return (display!=null);
    }
    
    /** Define quais textos sero utilizados pelo panel. */
    public void definirTextosDefault() {
        textos = new String[0];
    }
    
    /** Sorteia a posio dos ns de acordo com o tamanho atual do panel.
     */
    synchronized public void disporNosAleatoriamente() {
        if (display != null) {
            display.disporNosAleatoriamente();
        }
    }
    
    /** Sorteia a posio dos ns de acordo com uma dimenso especificada.
     * @param d Dimenso dentro da qual as posies dos ns devem ser sorteadas.
     */
    synchronized public void disporNosAleatoriamente(Dimension d) {
        if (display != null) {
            display.disporNosAleatoriamente(d);
        }
    }    
    
    /** Permite que os ns executem animaes.
     */
    public void habilitarAnimacao() {
        display.habilitarAnimacao();
    }

    /** No permite que os ns executem animaes.
     */
    public void desabilitarAnimacao() {
        display.desabilitarAnimacao();
    }

    /** Ajusta propriedades do grafo quando ele  exibido. 
     * @param componentEvent Evento.
     */
    public void componentShown(java.awt.event.ComponentEvent componentEvent) {
        display.ajustarLimitesDeEspacoDoGrafo();
    }
    
    /** No est sendo utilizado.
     * @param componentEvent Evento.
     */    
    public void componentMoved(java.awt.event.ComponentEvent componentEvent) {
    }
    
    /** No est sendo utilizado.
     * @param componentEvent Evento.
     */    
    public void componentResized(java.awt.event.ComponentEvent componentEvent) {
    }

    /** No est sendo utilizado.
     * @param componentEvent Evento.
     */    
    public void componentHidden(java.awt.event.ComponentEvent componentEvent) {
    }
        
}