/*
 * GrafoForceDirectedDisplay.java
 * Verso: 2004-08-24
 * Autor: Celmar Guimares da Silva 
 */

package grafo.forceDirected;
import java.awt.Graphics2D;
import java.awt.event.*;
import grafo.simples.GrafoSimplesDisplay;
import grafo.*;
import java.awt.geom.Point2D;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.util.Observer;

/** Panel em que o Grafo ser mostrado. Cuida de todos os cliques efetuados
 * nessa estrutura visual.
 */

public class GrafoForceDirectedDisplay extends GrafoSimplesDisplay 
implements MouseMotionListener, MouseListener, ActionListener, ComponentListener, Observer {
    
    /** Grafo que ser mostrado no display. */
    private GrafoForceDirected grafoFD;
    
    /** Mapa com as posies anteriores de cada n do grafo. 
     * Usado para possibilitar a animao do grafo para um estado estvel.
     */
    private Map posicoesAnteriores = new HashMap();
    
    /** Cria uma nova instncia de GrafoDisplay.
     * @param grafo Grafo a ser desenhado no display.
     */
    public GrafoForceDirectedDisplay(GrafoForceDirected grafo) {
        super(grafo);
        grafoFD = grafo;
        grafoFD.addObserver(this);
    }
    
    /** Desenha elementos que ficaro em segundo plano.
     * @param g Objeto Graphics em que o desenho ser feito.
     */
    public void desenhaElementosEmSegundoPlano(Graphics2D g) {
        // Nao existem elementos de segundo plano a serem desenhados...
    }

    /** Calcula quais as posies de cada n que faro o grafo ficar estvel
     * novamente, e faz uma animao dos ns para que alcancem essas posies.
     */
    synchronized public void animarGrafoParaEstadoEstavel() {

        if (grafoFD.autoAjusteEstaAtivo()) {
            posicoesAnteriores.clear();
            
            // Alimentar hashmap com ns e suas posies atuais
            // (que sero tidas como iniciais para o movimento)
            No n;
            PontoDuplo p;
            Iterator i;
            boolean devoAnimar = animacaoEstaHabilitada();
            
            
            if (devoAnimar) {
                i = grafoFD.nos.iterator();
                while (i.hasNext()) {
                    n = (No)i.next();
                    p = new PontoDuplo(new Point2D.Double(n.x, n.y), null);
                    posicoesAnteriores.put(n, p);
                }
            }
            
            // Procurando estabilidade
            grafoFD.procurarEstabilidade();
            grafoFD.centralizarGrafo();
            
            if (devoAnimar) {
                // Alimentando hashmap com posies finais calculadas pelo
                // algoritmo de aproximaao de vrtices, e movendo vrtices
                // para suas posies antigas.
                i = grafoFD.nos.iterator();
                while (i.hasNext()) {
                    n = (No)i.next();
                    p = (PontoDuplo)posicoesAnteriores.get(n);
                    p.fim = new Point2D.Double(n.x,n.y);
                    n.x = p.inicio.x;
                    n.y = p.inicio.y;
                }
                
                int quantidadeDePassos = 25;
                long tempoDeEspera = 600/quantidadeDePassos;
                double angulo = Math.PI;
                double deltaAngulo = Math.PI/quantidadeDePassos;
                for (int passo=0; passo<quantidadeDePassos; passo++) {
                    angulo += deltaAngulo;
                    for (int indiceNo=0; indiceNo<grafoFD.nos.size(); indiceNo++) {
                        n = (No)grafoFD.nos.get(indiceNo);
                        p = (PontoDuplo)posicoesAnteriores.get(n);
                        //                    n.x += (p.fim.x - p.inicio.x)/quantidadeDePassos;
                        //                    n.y += (p.fim.y - p.inicio.y)/quantidadeDePassos;
                        n.x = p.inicio.x + (1 + Math.cos(angulo))*(p.fim.x - p.inicio.x)/2;
                        n.y = p.inicio.y + (1 + Math.cos(angulo))*(p.fim.y - p.inicio.y)/2;
                    }
                    //repaint();
                    paintImmediately(0,0, getWidth(), getHeight());
                    try {
                        Thread.sleep(tempoDeEspera);
                    } catch (InterruptedException e) {
                        System.out.println("Espera interrompida.");
                    }
                }
            } 
            
        }
    }

    /** Mtodo que observa o grafo. Se ele for modificado, dispara o mtodo de
     * animao dos ns.
     * @param o Elemento observado. No  necessrio neste caso.
     * @param arg Argumento passado pelo elemento observado. No  necessrio 
     * neste caso.
     */
    public void update(java.util.Observable o, Object arg) {
        animarGrafoParaEstadoEstavel();
    }

}

/** Usado nesta classe para representar um conjunto de dois pontos com 
 * coordenadas x e y do tipo double.
 */
class PontoDuplo {
    /** Ponto inicial */
    Point2D.Double inicio = null;
    
    /** Ponto final */
    Point2D.Double fim = null;
    
    /** Cria uma nova instncia de PontoDuplo.
     * @param inicio Ponto inicial.
     * @param fim Ponto final.
     */
    public PontoDuplo(Point2D.Double inicio, Point2D.Double fim) {
        this.inicio = inicio;
        this.fim = fim;
    }
}